@@ -16,7 +16,12 @@ class getlang_Tests(unittest.TestCase):
         self.failUnlessEqual(_strptime._getlang(), locale.getlocale(locale.LC_TIME))
 
 class LocaleTime_Tests(unittest.TestCase):
-    """Tests for _strptime.LocaleTime."""
+    """Tests for _strptime.LocaleTime.
+
+    All values are lower-cased when stored in LocaleTime, so make sure to
+    compare values after running ``lower`` on them.
+
+    """
 
     def setUp(self):
         """Create time tuple based on current time."""
@@ -29,7 +34,7 @@ class LocaleTime_Tests(unittest.TestCase):
         tuple_position of time_tuple.  Uses error_msg as error message.
 
         """
-        strftime_output = time.strftime(directive, self.time_tuple)
+        strftime_output = time.strftime(directive, self.time_tuple).lower()
         comparison = testing[self.time_tuple[tuple_position]]
         self.failUnless(strftime_output in testing, "%s: not found in tuple" %
                                                     error_msg)
@@ -55,7 +60,7 @@ class LocaleTime_Tests(unittest.TestCase):
 
     def test_am_pm(self):
         # Make sure AM/PM representation done properly
-        strftime_output = time.strftime("%p", self.time_tuple)
+        strftime_output = time.strftime("%p", self.time_tuple).lower()
         self.failUnless(strftime_output in self.LT_ins.am_pm,
                         "AM/PM representation not in tuple")
         if self.time_tuple[3] < 12: position = 0
@@ -65,9 +70,10 @@ class LocaleTime_Tests(unittest.TestCase):
 
     def test_timezone(self):
         # Make sure timezone is correct
-        timezone = time.strftime("%Z", self.time_tuple)
+        timezone = time.strftime("%Z", self.time_tuple).lower()
         if timezone:
-            self.failUnless(timezone in self.LT_ins.timezone,
+            self.failUnless(timezone in self.LT_ins.timezone[0] or \
+                            timezone in self.LT_ins.timezone[1],
                             "timezone %s not found in %s" %
                             (timezone, self.LT_ins.timezone))
 
@@ -91,7 +97,8 @@ class LocaleTime_Tests(unittest.TestCase):
         self.failUnless(strftime_output == time.strftime(self.LT_ins.LC_time,
                                                          magic_date),
                         "LC_time incorrect")
-        LT = _strptime.LocaleTime(am_pm=('',''))
+        LT = _strptime.LocaleTime()
+        LT.am_pm = ('', '')
         self.failUnless(LT.LC_time, "LocaleTime's LC directives cannot handle "
                                     "empty strings")
 
@@ -100,44 +107,6 @@ class LocaleTime_Tests(unittest.TestCase):
         # Assuming locale has not changed between now and when self.LT_ins was created
         self.failUnlessEqual(self.LT_ins.lang, _strptime._getlang())
 
-    def test_by_hand_input(self):
-        # Test passed-in initialization value checks
-        self.failUnless(_strptime.LocaleTime(f_weekday=range(7)),
-                        "Argument size check for f_weekday failed")
-        self.assertRaises(TypeError, _strptime.LocaleTime, f_weekday=range(8))
-        self.assertRaises(TypeError, _strptime.LocaleTime, f_weekday=range(6))
-        self.failUnless(_strptime.LocaleTime(a_weekday=range(7)),
-                        "Argument size check for a_weekday failed")
-        self.assertRaises(TypeError, _strptime.LocaleTime, a_weekday=range(8))
-        self.assertRaises(TypeError, _strptime.LocaleTime, a_weekday=range(6))
-        self.failUnless(_strptime.LocaleTime(f_month=range(12)),
-                        "Argument size check for f_month failed")
-        self.assertRaises(TypeError, _strptime.LocaleTime, f_month=range(11))
-        self.assertRaises(TypeError, _strptime.LocaleTime, f_month=range(13))
-        self.failUnless(len(_strptime.LocaleTime(f_month=range(12)).f_month) == 13,
-                        "dummy value for f_month not added")
-        self.failUnless(_strptime.LocaleTime(a_month=range(12)),
-                        "Argument size check for a_month failed")
-        self.assertRaises(TypeError, _strptime.LocaleTime, a_month=range(11))
-        self.assertRaises(TypeError, _strptime.LocaleTime, a_month=range(13))
-        self.failUnless(len(_strptime.LocaleTime(a_month=range(12)).a_month) == 13,
-                        "dummy value for a_month not added")
-        self.failUnless(_strptime.LocaleTime(am_pm=range(2)),
-                        "Argument size check for am_pm failed")
-        self.assertRaises(TypeError, _strptime.LocaleTime, am_pm=range(1))
-        self.assertRaises(TypeError, _strptime.LocaleTime, am_pm=range(3))
-        self.failUnless(_strptime.LocaleTime(timezone=range(2)),
-                        "Argument size check for timezone failed")
-        self.assertRaises(TypeError, _strptime.LocaleTime, timezone=range(1))
-        self.assertRaises(TypeError, _strptime.LocaleTime, timezone=range(3))
-
-    def test_unknowntimezone(self):
-        # Handle timezone set to ('','') properly.
-        # Fixes bug #661354
-        locale_time = _strptime.LocaleTime(timezone=('',''))
-        self.failUnless("%Z" not in locale_time.LC_date,
-                        "when timezone == ('',''), string.replace('','%Z') is "
-                         "occuring")
 
 class TimeRETests(unittest.TestCase):
     """Tests for TimeRE."""
@@ -147,21 +116,6 @@ class TimeRETests(unittest.TestCase):
         self.time_re = _strptime.TimeRE()
         self.locale_time = _strptime.LocaleTime()
 
-    def test_getitem(self):
-        # Make sure that __getitem__ works properly
-        self.failUnless(self.time_re['m'],
-                        "Fetching 'm' directive (built-in) failed")
-        self.failUnless(self.time_re['b'],
-                        "Fetching 'b' directive (built w/ __tupleToRE) failed")
-        for name in self.locale_time.a_month:
-            self.failUnless(self.time_re['b'].find(name) != -1,
-                            "Not all abbreviated month names in regex")
-        self.failUnless(self.time_re['c'],
-                        "Fetching 'c' directive (built w/ format) failed")
-        self.failUnless(self.time_re['c'].find('%') == -1,
-                        "Conversion of 'c' directive failed; '%' found")
-        self.assertRaises(KeyError, self.time_re.__getitem__, '1')
-
     def test_pattern(self):
         # Test TimeRE.pattern
         pattern_string = self.time_re.pattern(r"%a %A %d")
@@ -212,7 +166,8 @@ class TimeRETests(unittest.TestCase):
     def test_blankpattern(self):
         # Make sure when tuple or something has no values no regex is generated.
         # Fixes bug #661354
-        test_locale = _strptime.LocaleTime(timezone=('',''))
+        test_locale = _strptime.LocaleTime()
+        test_locale.timezone = (frozenset(), frozenset())
         self.failUnless(_strptime.TimeRE(test_locale).pattern("%Z") == '',
                         "with timezone == ('',''), TimeRE().pattern('%Z') != ''")
 
@@ -222,6 +177,28 @@ class TimeRETests(unittest.TestCase):
         found = compiled_re.match("\w+ 10")
         self.failUnless(found, "Escaping failed of format '\w+ 10'")
 
+    def test_locale_data_w_regex_metacharacters(self):
+        # Check that if locale data contains regex metacharacters they are
+        # escaped properly.
+        # Discovered by bug #1039270 .
+        locale_time = _strptime.LocaleTime()
+        locale_time.timezone = (frozenset(("utc", "gmt",
+                                            "Tokyo (standard time)")),
+                                frozenset("Tokyo (daylight time)"))
+        time_re = _strptime.TimeRE(locale_time)
+        self.failUnless(time_re.compile("%Z").match("Tokyo (standard time)"),
+                        "locale data that contains regex metacharacters is not"
+                        " properly escaped")
+
+    def test_whitespace_substitution(self):
+        # When pattern contains whitespace, make sure it is taken into account
+        # so as to not allow to subpatterns to end up next to each other and
+        # "steal" characters from each other.
+        pattern = self.time_re.pattern('%j %H')
+        self.failUnless(not re.match(pattern, "180"))
+        self.failUnless(re.match(pattern, "18 0"))
+
+
 class StrptimeTests(unittest.TestCase):
     """Tests for _strptime.strptime."""
 
@@ -229,19 +206,29 @@ class StrptimeTests(unittest.TestCase):
         """Create testing time tuple."""
         self.time_tuple = time.gmtime()
 
-    def test_TypeError(self):
-        # Make sure ValueError is raised when match fails
-        self.assertRaises(ValueError, _strptime.strptime, data_string="%d",
+    def test_ValueError(self):
+        # Make sure ValueError is raised when match fails or format is bad
+        self.assertRaises(ValueError, _strptime._strptime_time, data_string="%d",
                           format="%A")
+        for bad_format in ("%", "% ", "%e"):
+            try:
+                _strptime._strptime_time("2005", bad_format)
+            except ValueError:
+                continue
+            except Exception, err:
+                self.fail("'%s' raised %s, not ValueError" %
+                            (bad_format, err.__class__.__name__))
+            else:
+                self.fail("'%s' did not raise ValueError" % bad_format)
 
     def test_unconverteddata(self):
         # Check ValueError is raised when there is unconverted data
-        self.assertRaises(ValueError, _strptime.strptime, "10 12", "%m")
+        self.assertRaises(ValueError, _strptime._strptime_time, "10 12", "%m")
 
     def helper(self, directive, position):
         """Helper fxn in testing."""
         strf_output = time.strftime("%" + directive, self.time_tuple)
-        strp_output = _strptime.strptime(strf_output, "%" + directive)
+        strp_output = _strptime._strptime_time(strf_output, "%" + directive)
         self.failUnless(strp_output[position] == self.time_tuple[position],
                         "testing of '%s' directive failed; '%s' -> %s != %s" %
                          (directive, strf_output, strp_output[position],
@@ -254,7 +241,7 @@ class StrptimeTests(unittest.TestCase):
         # Must also make sure %y values are correct for bounds set by Open Group
         for century, bounds in ((1900, ('69', '99')), (2000, ('00', '68'))):
             for bound in bounds:
-                strp_output = _strptime.strptime(bound, '%y')
+                strp_output = _strptime._strptime_time(bound, '%y')
                 expected_result = century + int(bound)
                 self.failUnless(strp_output[0] == expected_result,
                                 "'y' test failed; passed in '%s' "
@@ -273,7 +260,7 @@ class StrptimeTests(unittest.TestCase):
         # Test hour directives
         self.helper('H', 3)
         strf_output = time.strftime("%I %p", self.time_tuple)
-        strp_output = _strptime.strptime(strf_output, "%I %p")
+        strp_output = _strptime._strptime_time(strf_output, "%I %p")
         self.failUnless(strp_output[3] == self.time_tuple[3],
                         "testing of '%%I %%p' directive failed; '%s' -> %s != %s" %
                          (strf_output, strp_output[3], self.time_tuple[3]))
@@ -286,6 +273,13 @@ class StrptimeTests(unittest.TestCase):
         # Test second directives
         self.helper('S', 5)
 
+    def test_fraction(self):
+        # Test microseconds
+        import datetime
+        d = datetime.datetime(2012, 12, 20, 12, 34, 56, 78987)
+        tup, frac = _strptime._strptime(str(d), format="%Y-%m-%d %H:%M:%S.%f")
+        self.assertEqual(frac, d.microsecond)
+
     def test_weekday(self):
         # Test weekday directives
         for directive in ('A', 'a', 'w'):
@@ -300,17 +294,17 @@ class StrptimeTests(unittest.TestCase):
         # When gmtime() is used with %Z, entire result of strftime() is empty.
         # Check for equal timezone names deals with bad locale info when this
         # occurs; first found in FreeBSD 4.4.
-        strp_output = _strptime.strptime("UTC", "%Z")
+        strp_output = _strptime._strptime_time("UTC", "%Z")
         self.failUnlessEqual(strp_output.tm_isdst, 0)
-        strp_output = _strptime.strptime("GMT", "%Z")
+        strp_output = _strptime._strptime_time("GMT", "%Z")
         self.failUnlessEqual(strp_output.tm_isdst, 0)
-        time_tuple = time.localtime()
-        strf_output = time.strftime("%Z")  #UTC does not have a timezone
-        strp_output = _strptime.strptime(strf_output, "%Z")
-        locale_time = _strptime.LocaleTime()
-        if sys.platform == 'mac':
+        if sys.platform == "mac":
             # Timezones don't really work on MacOS9
             return
+        time_tuple = time.localtime()
+        strf_output = time.strftime("%Z")  #UTC does not have a timezone
+        strp_output = _strptime._strptime_time(strf_output, "%Z")
+        locale_time = _strptime.LocaleTime()
         if time.tzname[0] != time.tzname[1] or not time.daylight:
             self.failUnless(strp_output[8] == time_tuple[8],
                             "timezone check failed; '%s' -> %s != %s" %
@@ -324,24 +318,23 @@ class StrptimeTests(unittest.TestCase):
         # Explicitly test possibility of bad timezone;
         # when time.tzname[0] == time.tzname[1] and time.daylight
         if sys.platform == "mac":
-            return # MacOS9 has severely broken timezone support.
+            return #MacOS9 has severely broken timezone support.
+        tz_name = time.tzname[0]
+        if tz_name.upper() in ("UTC", "GMT"):
+            return
         try:
             original_tzname = time.tzname
             original_daylight = time.daylight
-            time.tzname = ("PDT", "PDT")
+            time.tzname = (tz_name, tz_name)
             time.daylight = 1
-            # Need to make sure that timezone is not calculated since that
-            # calls time.tzset and overrides temporary changes to time .
-            _strptime._locale_cache = _strptime.TimeRE(_strptime.LocaleTime(
-                                                    timezone=("PDT", "PDT")))
-            _strptime._regex_cache.clear()
-            tz_value = _strptime.strptime("PDT", "%Z")[8]
-            self.failUnlessEqual(tz_value, -1)
+            tz_value = _strptime._strptime_time(tz_name, "%Z")[8]
+            self.failUnlessEqual(tz_value, -1,
+                    "%s lead to a timezone value of %s instead of -1 when "
+                    "time.daylight set to %s and passing in %s" %
+                    (time.tzname, tz_value, time.daylight, tz_name))
         finally:
             time.tzname = original_tzname
             time.daylight = original_daylight
-            _strptime._locale_cache = _strptime.TimeRE()
-            _strptime._regex_cache.clear()
 
     def test_date_time(self):
         # Test %c directive
@@ -361,7 +354,7 @@ class StrptimeTests(unittest.TestCase):
     def test_percent(self):
         # Make sure % signs are handled properly
         strf_output = time.strftime("%m %% %Y", self.time_tuple)
-        strp_output = _strptime.strptime(strf_output, "%m %% %Y")
+        strp_output = _strptime._strptime_time(strf_output, "%m %% %Y")
         self.failUnless(strp_output[0] == self.time_tuple[0] and
                          strp_output[1] == self.time_tuple[1],
                         "handling of percent sign failed")
@@ -369,17 +362,17 @@ class StrptimeTests(unittest.TestCase):
     def test_caseinsensitive(self):
         # Should handle names case-insensitively.
         strf_output = time.strftime("%B", self.time_tuple)
-        self.failUnless(_strptime.strptime(strf_output.upper(), "%B"),
+        self.failUnless(_strptime._strptime_time(strf_output.upper(), "%B"),
                         "strptime does not handle ALL-CAPS names properly")
-        self.failUnless(_strptime.strptime(strf_output.lower(), "%B"),
+        self.failUnless(_strptime._strptime_time(strf_output.lower(), "%B"),
                         "strptime does not handle lowercase names properly")
-        self.failUnless(_strptime.strptime(strf_output.capitalize(), "%B"),
+        self.failUnless(_strptime._strptime_time(strf_output.capitalize(), "%B"),
                         "strptime does not handle capword names properly")
 
     def test_defaults(self):
         # Default return value should be (1900, 1, 1, 0, 0, 0, 0, 1, 0)
         defaults = (1900, 1, 1, 0, 0, 0, 0, 1, -1)
-        strp_output = _strptime.strptime('1', '%m')
+        strp_output = _strptime._strptime_time('1', '%m')
         self.failUnless(strp_output == defaults,
                         "Default values for strptime() are incorrect;"
                         " %s != %s" % (strp_output, defaults))
@@ -391,7 +384,7 @@ class StrptimeTests(unittest.TestCase):
         # escaped.
         # Test instigated by bug #796149 .
         need_escaping = ".^$*+?{}\[]|)("
-        self.failUnless(_strptime.strptime(need_escaping, need_escaping))
+        self.failUnless(_strptime._strptime_time(need_escaping, need_escaping))
 
 class Strptime12AMPMTests(unittest.TestCase):
     """Test a _strptime regression in '%I %p' at 12 noon (12 PM)"""
@@ -400,8 +393,8 @@ class Strptime12AMPMTests(unittest.TestCase):
         eq = self.assertEqual
         eq(time.strptime('12 PM', '%I %p')[3], 12)
         eq(time.strptime('12 AM', '%I %p')[3], 0)
-        eq(_strptime.strptime('12 PM', '%I %p')[3], 12)
-        eq(_strptime.strptime('12 AM', '%I %p')[3], 0)
+        eq(_strptime._strptime_time('12 PM', '%I %p')[3], 12)
+        eq(_strptime._strptime_time('12 AM', '%I %p')[3], 0)
 
 
 class JulianTests(unittest.TestCase):
@@ -411,7 +404,7 @@ class JulianTests(unittest.TestCase):
         eq = self.assertEqual
         for i in range(1, 367):
             # use 2004, since it is a leap year, we have 366 days
-            eq(_strptime.strptime('%d 2004' % i, '%j %Y')[7], i)
+            eq(_strptime._strptime_time('%d 2004' % i, '%j %Y')[7], i)
 
 class CalculationTests(unittest.TestCase):
     """Test that strptime() fills in missing info correctly"""
@@ -422,7 +415,7 @@ class CalculationTests(unittest.TestCase):
     def test_julian_calculation(self):
         # Make sure that when Julian is missing that it is calculated
         format_string = "%Y %m %d %H %M %S %w %Z"
-        result = _strptime.strptime(time.strftime(format_string, self.time_tuple),
+        result = _strptime._strptime_time(time.strftime(format_string, self.time_tuple),
                                     format_string)
         self.failUnless(result.tm_yday == self.time_tuple.tm_yday,
                         "Calculation of tm_yday failed; %s != %s" %
@@ -431,7 +424,7 @@ class CalculationTests(unittest.TestCase):
     def test_gregorian_calculation(self):
         # Test that Gregorian date can be calculated from Julian day
         format_string = "%Y %H %M %S %w %j %Z"
-        result = _strptime.strptime(time.strftime(format_string, self.time_tuple),
+        result = _strptime._strptime_time(time.strftime(format_string, self.time_tuple),
                                     format_string)
         self.failUnless(result.tm_year == self.time_tuple.tm_year and
                          result.tm_mon == self.time_tuple.tm_mon and
@@ -445,7 +438,7 @@ class CalculationTests(unittest.TestCase):
     def test_day_of_week_calculation(self):
         # Test that the day of the week is calculated as needed
         format_string = "%Y %m %d %H %S %j %Z"
-        result = _strptime.strptime(time.strftime(format_string, self.time_tuple),
+        result = _strptime._strptime_time(time.strftime(format_string, self.time_tuple),
                                     format_string)
         self.failUnless(result.tm_wday == self.time_tuple.tm_wday,
                         "Calculation of day of the week failed;"
@@ -459,7 +452,7 @@ class CalculationTests(unittest.TestCase):
                 format_string = "%%Y %%%s %%w" % directive
                 dt_date = datetime_date(*ymd_tuple)
                 strp_input = dt_date.strftime(format_string)
-                strp_output = _strptime.strptime(strp_input, format_string)
+                strp_output = _strptime._strptime_time(strp_input, format_string)
                 self.failUnless(strp_output[:3] == ymd_tuple,
                         "%s(%s) test failed w/ '%s': %s != %s (%s != %s)" %
                             (test_reason, directive, strp_input,
@@ -486,6 +479,10 @@ class CalculationTests(unittest.TestCase):
                                         "of the year")
         test_helper((1917, 12, 31), "Dec 31 on Monday with year starting and "
                                         "ending on Monday")
+        test_helper((2007, 01, 07), "First Sunday of 2007")
+        test_helper((2007, 01, 14), "Second Sunday of 2007")
+        test_helper((2006, 12, 31), "Last Sunday of 2006")
+        test_helper((2006, 12, 24), "Second to last Sunday of 2006")
 
 
 class CacheTests(unittest.TestCase):
@@ -494,11 +491,13 @@ class CacheTests(unittest.TestCase):
     def test_time_re_recreation(self):
         # Make sure cache is recreated when current locale does not match what
         # cached object was created with.
-        _strptime.strptime("10", "%d")
-        _strptime._locale_cache.locale_time = _strptime.LocaleTime(lang="Ni")
-        original_time_re = id(_strptime._locale_cache)
-        _strptime.strptime("10", "%d")
-        self.failIfEqual(original_time_re, id(_strptime._locale_cache))
+        _strptime._strptime_time("10", "%d")
+        _strptime._strptime_time("2005", "%Y")
+        _strptime._TimeRE_cache.locale_time.lang = "Ni"
+        original_time_re = id(_strptime._TimeRE_cache)
+        _strptime._strptime_time("10", "%d")
+        self.failIfEqual(original_time_re, id(_strptime._TimeRE_cache))
+        self.failUnlessEqual(len(_strptime._regex_cache), 1)
 
     def test_regex_cleanup(self):
         # Make sure cached regexes are discarded when cache becomes "full".
@@ -507,24 +506,49 @@ class CacheTests(unittest.TestCase):
         except KeyError:
             pass
         bogus_key = 0
-        while len(_strptime._regex_cache) <= 5:
+        while len(_strptime._regex_cache) <= _strptime._CACHE_MAX_SIZE:
             _strptime._regex_cache[bogus_key] = None
             bogus_key += 1
-        _strptime.strptime("10", "%d")
+        _strptime._strptime_time("10", "%d")
         self.failUnlessEqual(len(_strptime._regex_cache), 1)
 
     def test_new_localetime(self):
         # A new LocaleTime instance should be created when a new TimeRE object
         # is created.
-        _strptime._locale_cache.locale_time = _strptime.LocaleTime(lang="Ni")
-        locale_time_id = id(_strptime._locale_cache.locale_time)
-        locale_time_lang = _strptime._locale_cache.locale_time.lang
-        _strptime.strptime("10", "%d")
+        locale_time_id = id(_strptime._TimeRE_cache.locale_time)
+        _strptime._TimeRE_cache.locale_time.lang = "Ni"
+        _strptime._strptime_time("10", "%d")
         self.failIfEqual(locale_time_id,
-                         id(_strptime._locale_cache.locale_time))
-        self.failIfEqual(locale_time_lang,
-                         _strptime._locale_cache.locale_time.lang)
+                         id(_strptime._TimeRE_cache.locale_time))
 
+    def test_TimeRE_recreation(self):
+        # The TimeRE instance should be recreated upon changing the locale.
+        locale_info = locale.getlocale(locale.LC_TIME)
+        try:
+            locale.setlocale(locale.LC_TIME, ('en_US', 'UTF8'))
+        except locale.Error:
+            return
+        try:
+            _strptime._strptime_time('10', '%d')
+            # Get id of current cache object.
+            first_time_re_id = id(_strptime._TimeRE_cache)
+            try:
+                # Change the locale and force a recreation of the cache.
+                locale.setlocale(locale.LC_TIME, ('de_DE', 'UTF8'))
+                _strptime._strptime_time('10', '%d')
+                # Get the new cache object's id.
+                second_time_re_id = id(_strptime._TimeRE_cache)
+                # They should not be equal.
+                self.failIfEqual(first_time_re_id, second_time_re_id)
+            # Possible test locale is not supported while initial locale is.
+            # If this is the case just suppress the exception and fall-through
+            # to the reseting to the original locale.
+            except locale.Error:
+                pass
+        # Make sure we don't trample on the locale setting once we leave the
+        # test.
+        finally:
+            locale.setlocale(locale.LC_TIME, locale_info)
 
 
 def test_main():
