@@ -4,7 +4,7 @@
 
 __version__ = "special BeOS after 1.37"
 
-import sys, os, getopt
+import sys, os
 from distutils import sysconfig
 from distutils import text_file
 from distutils.errors import *
@@ -176,8 +176,6 @@ class PyBuildExt(build_ext):
         #
 
         # Some modules that are normally always on:
-        exts.append( Extension('regex', ['regexmodule.c', 'regexpr.c']) )
-
         exts.append( Extension('_weakref', ['_weakref.c']) )
         exts.append( Extension('_symtable', ['symtablemodule.c']) )
 
@@ -197,7 +195,7 @@ class PyBuildExt(build_ext):
                                libraries=math_libs) )
         # operator.add() and similar goodies
         exts.append( Extension('operator', ['operator.c']) )
-        # access to the builtin codecs and codec registry
+        # access to the built-in codecs and codec registry
         exts.append( Extension('_codecs', ['_codecsmodule.c']) )
         # Python C API test module
         exts.append( Extension('_testcapi', ['_testcapimodule.c']) )
